const URL = "/api/hackData";
const RELOAD_INTERVAL = 1000;
let hackData;

window.acquire = (id) => {
    const target = selectTarget(id);
    if (target == null) {
        return;
    }
    if (hackData.point < target.point) {
        alert(`ポイントが足りません。\n\n${target.title}\n\n${target.description}`);
        return;
    }
    if (!confirm(`${target.point}pt を使用し「${target.title}」を取得します。\n\n${target.description}`)) {
        return;
    }
    fetch(URL, {
        method: "POST",
        headers: {
            Accept: "application/json",
            "Content-Type": "application/json",
        },
        body: JSON.stringify({
            acquire: id,
        }),
    });
};
window.showDetail = (id) => {
    const target = selectTarget(id);
    if (target == null) {
        return;
    }
    confirm(`${target.title}\n\n${target.description}`);
};

const selectTarget = (id) => {
    if (hackData.tree == null) {
        return null;
    }
    for (const tree of hackData.tree) {
        if (tree && tree.hasOwnProperty(id)) {
            return tree[id];
        }
    }
    return null;
};

const createMermaidData = (data) => {
    const head = "graph TD;";
    if (!data) {
        return head;
    }
    return (
        head +
        Object.keys(data)
            .map((key) => {
                const d = data[key];
                const color = d.acquired ? "skyblue" : d.available ? "lightgreen" : "lightgray";
                const hasAnotherRoute = d.parents.length >= 2;
                const arrows = d.parents
                    .map((ps, index) => {
                        return ps
                            .map((p) => {
                                if (hasAnotherRoute) {
                                    return `${p}--Route ${index + 1} -->${key}`;
                                }
                                return `${p}-->${key}`;
                            })
                            .join(";");
                    })
                    .join(";");
                return `${key}(${d.title}${(!d.acquired && d.point > 0 && "\n" + d.point + "pt") || ""});` + `style ${key} fill:${color},stroke:gray;` + `click ${key} ${!d.acquired && d.available ? "acquire" : "showDetail"} "${d.description}";` + arrows;
            })
            .join(";")
    );
};

const renderTree = () => {
    const base = document.querySelector("#tree");
    const scrollTop = base.scrollTop;
    const scrollLeft = base.scrollLeft;

    // 削除
    while (base.firstChild) {
        base.removeChild(base.firstChild);
    }
    // 追加
    hackData.tree.forEach((tree) => {
        const elm = document.createElement("div");
        elm.className = "mermaid";
        elm.innerText = createMermaidData(tree);
        base.appendChild(elm);
    });

    mermaid.init();
    document.querySelectorAll("#tree .mermaid").forEach((elm) => {
        elm.style.width = elm.querySelector("svg").style["max-width"];
    });

    base.scrollTop = scrollTop;
    base.scrollLeft = scrollLeft;
};

const renderData = () => {
    const panelElm = document.createElement("ul");
    hackData.tree.forEach((tree) => {
        Object.keys(tree).map((key) => {
            if (!tree[key].acquired) {
                return;
            }
            const elm = document.createElement("li");
            elm.appendChild(document.createTextNode(`≪${tree[key].title}≫`));
            elm.appendChild(document.createElement("br"));
            elm.appendChild(document.createTextNode(tree[key].description));
            panelElm.appendChild(elm);
        });
    });

    const pointElm = document.createElement("p");
    pointElm.appendChild(document.createTextNode(`所持ポイント : ${hackData.point} pt`));

    const dataElm = document.querySelector("#data");
    const scrollTop = dataElm.scrollTop;
    while (dataElm.firstChild) {
        dataElm.removeChild(dataElm.firstChild);
    }
    dataElm.appendChild(pointElm);
    dataElm.appendChild(panelElm);

    dataElm.scrollTop = scrollTop;
};

const render = () => {
    renderTree();
    renderData();
};

const getHackData = () => {
    return fetch(URL)
        .then((response) => {
            if (response.status != 200) {
                throw Error(response.statusText);
            }
            return response.json();
        })
        .then((jsonResponse) => {
            if (hackData && hackData.id === jsonResponse.id) {
                return false;
            }
            hackData = jsonResponse;
            return true;
        });
};

const start = () => {
    getHackData()
        .then((refresh) => {
            if (refresh) {
                render();
            }
            setTimeout(start, RELOAD_INTERVAL);
        })
        .catch((e) => {
            alert(e);
            console.error(e);
        });
};

window.addEventListener("load", start);
